@@ -1,12 +1,230 @@
 """
-Lisz Demo App main app base (ae.gui_app) mixin class
-====================================================
+GUI framework implementations
+=============================
 
-This module is used for to provide common data structures, functions and methods for the various
-demo applications based on the gui_app and the related GUI framework implementation portions
-(like e.g. ae.kivy_app and ae.enaml_app) of the ae namespace.
+This module is used as showcase base for the implementation of demo
+applications of the GUI framework packages provided
+by the ae namespace.
+
+The plan is to integrate the following GUI frameworks on
+top of the :class:`abstract base class <~ae.gui_app.MainAppBase>`
+(implemented in the :mod:`ae.gui_app` portion of the
+ae namespace):
+
+* :mod:`Kivy <ae.kivy_app>` - see also :ref:`kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`
+* :mod:`Enaml <ae.enaml_app>` - see also :ref:`enaml lisz demo app <https://gitlab.com/ae-group/enaml_lisz>`
+* :mod:`Beeware <ae.beeware_app>`
+* :mod:`QPython <ae.qpython_app>`
+* :mod:`pyglet <ae.pyglet_app>`
+* :mod:`pygobject <ae.pygobject_app>`
+* :mod:`Dabo <ae.dabo_app>`
+* :mod:`AppJar <ae.appjar_app>`
+
+The main app base mixin class :class:`LiszDataMixin` provided by this module
+is used for to manage the common data structures, functions and methods
+for the various demo applications variants based on :mod:`ae.gui_app`
+and the related GUI framework implementation portions
+(like e.g. :mod:`ae.kivy_app` and :mod:`ae.enaml_app`) of the ae namespace.
+
+Further down in this module you find additional notes on each
+of the already implemented GUI frameworks.
+
+
+Lisz Demo App
+=============
+
+The Lisz demo app is managing a recursive item tree that can be used
+e.g. as to-do or shopping list.
+
+
+features of the Lisz demo app
+-----------------------------
+
+* recursive item data tree manipulation: add, edit and delete item
+* each item can be selected/check marked
+* easy filtering of selected/checked and unselected/unchecked items
+* each item can be converted to a sub-node (sub-list) and back to a leaf item
+* item order changeable via drag & drop
+* item movable into parent and sub node
+* easy navigation within the item tree (up/down navigation in tree and quick jump)
+* simple demo help screen
+* colors changeable by user
+* font and button sizes are changeable by user
+* dark and light theme switchable by user
+* sound output support with sound volume configurable by user
+* vibration output support with amplitude configurable by user (only for mobile apps)
+
+
+lisz application data model
+---------------------------
+
+To keep it simply, the data managed by the Lisz application is
+a minimalistic tree structure that gets stored as
+an :ref:`application status`. The root node and with that
+the whole recursive data structure
+gets stored in the app state variable `root_node`,
+without the need of any database.
+
+The root of the tree structure is a list of the type `LiszNode`
+containing list items of type `LiszItem`. A `LiszItem` element
+represents a dict of the type `Dict[str, Any]`.
+
+Each `LiszItem` element of the tree structure is either a
+leaf or a node. And each node is a sub-list with a recursive
+structure identical to the root node and of the type `LiszNode`.
+
+The following graph is showing an example tree:
+
+.. graphviz::
+
+    digraph {
+        node [shape=record, width=3]
+        rec1 [label="{<rec1>Root Node | { <A>Item A | <C>Item C | <D>... } }"]
+        "root_node app state variable" -> rec1 [arrowhead=crow style=tapered penwidth=3]
+        rec1:A -> "Leaf Item A" [minlen=3]
+        rec2 [label="{<rec2>Node Item C (sub-node) | { <CA>Item CA | <CB>Item CB | <CN>... } }"]
+        rec1:C -> rec2
+        rec2:CA -> "Leaf Item CA" [minlen=2]
+        rec3 [label="{<rec3>Node Item CB (sub-sub-node) | { <CBA>Item CBA | <CDn>... } }"]
+        rec2:CB -> rec3
+        rec3:CBA -> "Leaf Item CBA"
+    }
+
+The above example tree structure is containing
+the root node items `A` -which is a leaf - and `C` - which is
+a sub-node.
+
+The node `C` consists of the items `CA` and `CB` where `CA` is
+a leaf and `CB` is a node.
+
+Finally the first item of the node `CB` is another
+sub-node with the leaf item `CBA`.
+
+
+gui framework implementation variants
+=====================================
+
+kivy
+----
+
+The :ref:`kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>` is based
+on the :ref:`kivy framework <https://kivy.org>`,
+a :ref:`pypi package <https://pypi.org/project/Kivy/>`
+documented :ref:`here <https://kivy.org/doc/stable/>`.
+
+
+enaml
+-----
+
+The :ref:`enaml lisz demo app <https://gitlab.com/ae-group/enaml_lisz>` is
+based on the :ref:`enaml framework <https://pypi.org/project/enaml/>`,
+a :ref:`pypi package <https://pypi.org/project/enaml/>`
+documented :ref:`here at ReadTheDocs <https://enaml.readthedocs.io/en/latest/>`.
+
+
+automatic update of widget attributes
+_____________________________________
+
+Dependencies have to be executed/read_from, so e.g. the icon attribute will not be updated if
+app.ae_state_light_theme gets changed::
+
+    icon << main_app.cached_icon('font_size') or app.ae_state_light_theme
+
+In contrary the icon will be updated by the following two statements::
+
+    icon << main_app.cached_icon('font_size') if app.ae_state_light_theme else main_app.cached_icon('font_size')
+    icon << app.ae_state_light_theme == None or main_app.cached_icon('font_size')
+
+KeyEvent implementation based on this SO answer posted by the enamlx author frmdstryr/Jairus Martin:
+https://stackoverflow.com/questions/20380940/how-to-get-key-events-when-using-enaml. Alternative
+and more complete implementation can be found in the enamlx package (https://github.com/frmdstryr/enamlx).
+
+
+which list
+__________
+
+* type and syntax checking, code highlighting and debugging of enaml files within PyCharm.
+* fix freezing of linux/Ubuntu system in debugging of opening/opened PopupViews in PyCharm
+  (killing of java PyCharm process helps).
+* fix strange crashes after intensive usage of app (e.g. after pressing TAB or DOWN ARROW key 15-21 times)::
+
+    * python.exe 3.6.9::
+
+        * Segmentation fault (core dumped)
+        * EnamlMainApp.on_flow_widget_focused() 'focus_item:TUtu' <main_view.ThemeButton object at 0x7f891c0d4a48>
+                                                                                                          hasFocus=True
+            Traceback (most recent call last):
+              File "/home/andi/.pyenv/versions/enaml_env/lib/python3.6/site-packages/enaml/qt/qt_container.py", line 507
+                                                                                                 , in _on_relayout_timer
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/contextlib.py", line 159, in helper
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/contextlib.py", line 63, in __init__
+            NameError: name 'getattr' is not defined
+            Unhandled Python exception
+            Aborted (core dumped)
+
+    * PyCharm 3.6.9 Run and miniconda 3.7 (Run or Debug)::
+
+        * process finished with exit code 139 (interrupted by signal 11: SIGSEGV)
+
+    * PyCharm 3.6.9 Debug::
+
+        *   Fatal Python error: Cannot recover from stack overflow::
+
+            Thread 0x00007f1865ec4700 (most recent call first):
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 299 in wait
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 551 in wait
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd.py", line 144 in _on_run
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/_pydevd_bundle/pydevd_comm.py", line 215
+                                                                                                                  in run
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 916 in _bootstrap_inner
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 884 in _bootstrap
+
+            Thread 0x00007f18666c5700 (most recent call first):
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/_pydevd_bundle/pydevd_comm.py", line 285
+                                                                                                              in _on_run
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/_pydevd_bundle/pydevd_comm.py", line 215
+                                                                                                                  in run
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 916 in _bootstrap_inner
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 884 in _bootstrap
+
+            Thread 0x00007f1866ec6700 (most recent call first):
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 299 in wait
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/queue.py", line 173 in get
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/_pydevd_bundle/pydevd_comm.py", line 360
+                                                                                                              in _on_run
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/_pydevd_bundle/pydevd_comm.py", line 215
+                                                                                                                  in run
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 916 in _bootstrap_inner
+              File "/home/andi/.pyenv/versions/3.6.9/lib/python3.6/threading.py", line 884 in _bootstrap
+
+            Current thread 0x00007f18722bc080 (most recent call first):
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd_file_utils.py", line 545
+                                                                                                              in _is_int
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd_file_utils.py", line 552
+                                                                                                         in is_real_file
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd_file_utils.py", line 568
+                                                                            in get_abs_path_real_path_and_base_from_file
+              File "/home/andi/.pyenv/versions/enaml_env/lib/python3.6/site-packages/enaml/qt/qt_widget.py", line 182
+                                                                                                    in tab_focus_request
+              File "/home/andi/.pyenv/versions/enaml_env/lib/python3.6/site-packages/enaml/qt/qt_widget.py", line 242
+                                                                                                   in focusNextPrevChild
+              File "/home/andi/.pyenv/versions/enaml_env/lib/python3.6/site-packages/enaml/qt/qt_application.py", line
+                                                                                                             48 in start
+              File "/home/andi/src/enaml_lisz/ae/enaml_app/__init__.py", line 134 in _event_loop_start
+              File "/home/andi/src/enaml_lisz/ae/gui_app.py", line 828 in run_app
+              File "/home/andi/src/enaml_lisz/main.py", line 346 in <module>
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/_pydev_imps/_pydev_execfile.py", line 18
+                                                                                                             in execfile
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd.py", line 1438 in _exec
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd.py", line 1431 in run
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd.py", line 2122 in main
+              File "/snap/pycharm-community/192/plugins/python-ce/helpers/pydev/pydevd.py", line 2131 in <module>
+
+            Process finished with exit code 134 (interrupted by signal 6: SIGABRT)
+
+
+- TODO: add unit tests for 100 % coverage.
 
-TODO: add unit tests for 100 % coverage.
 """
 from abc import abstractmethod, ABC
 from typing import Any, Dict, List, Optional, Sequence
@@ -15,7 +233,7 @@ from ae.core import DEBUG_LEVEL_VERBOSE
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action       # type: ignore
 
 
-__version__ = '0.0.5'
+__version__ = '0.0.6'
 
 
 FLOW_PATH_ROOT_ID = '<<<root>>>'     #: needed for flow path jumper and for drop onto leave item button
